# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/65_medical.text.ipynb (unless otherwise specified).


from __future__ import annotations


__all__ = []

# Cell
#nbdev_comment from __future__ import annotations
from ..basics import *